<?php

namespace Drupal\path_watcher\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element\Checkboxes;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Configure path watcher settings for this site.
 */
class PathWatcherSettingsForm extends ConfigFormBase {

  /**
   * The module config name.
   */
  const CONFIG_NAME = 'path_watcher.settings';

  /**
   * Constructs the object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   * @param \Drupal\Core\Config\TypedConfigManagerInterface $typedConfigManager
   *   The typed config manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(
    ConfigFactoryInterface $configFactory,
    TypedConfigManagerInterface $typedConfigManager,
    protected EntityTypeManagerInterface $entityTypeManager,
  ) {
    parent::__construct($configFactory, $typedConfigManager);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('config.typed'),
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'path_watcher_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [static::CONFIG_NAME];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(static::CONFIG_NAME)->get();
    $form['skip_admin_paths'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Skip admin paths'),
      '#default_value' => $config['skip_admin_paths'],
    ];
    $skip_usernames = $config['skip_usernames'];
    $form['skip_usernames'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Skip usernames'),
      '#description' => $this->t('One username per line. Skip visits with these usernames.'),
      '#default_value' => $skip_usernames ? implode(PHP_EOL, $skip_usernames) : '',
      '#rows' => min(5, count($skip_usernames) + 1),
    ];
    $roles = [];
    foreach ($this->entityTypeManager->getStorage('user_role')->loadMultiple() as $role) {
      $roles[$role->id()] = $role->label();
    }
    $skip_roles = $config['skip_roles'];
    $form['skip_roles_container'] = [
      '#type' => 'details',
      '#title' => $this->t('Skip roles (@count)', [
        '@count' => count($skip_roles),
      ]),
      '#open' => FALSE,
    ];
    $form['skip_roles_container']['skip_roles'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Skip roles'),
      '#title_display' => 'invisible',
      '#default_value' => $skip_roles,
      '#options' => $roles,
    ];

    $form['advanced'] = [
      '#type' => 'details',
      '#title' => $this->t('Advanced'),
      '#open' => FALSE,
    ];
    $form['advanced']['skip_xml_http_request'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Skip XML HTTP requests'),
      '#default_value' => $config['skip_xml_http_request'],
    ];
    $form['advanced']['methods'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Methods'),
      '#options' => [
        Request::METHOD_HEAD => Request::METHOD_HEAD,
        Request::METHOD_GET => Request::METHOD_GET,
        Request::METHOD_POST => Request::METHOD_POST,
        Request::METHOD_PUT => Request::METHOD_PUT,
        Request::METHOD_PATCH => Request::METHOD_PATCH,
        Request::METHOD_DELETE => Request::METHOD_DELETE,
        Request::METHOD_PURGE => Request::METHOD_PURGE,
        Request::METHOD_OPTIONS => Request::METHOD_OPTIONS,
        Request::METHOD_TRACE => Request::METHOD_TRACE,
        Request::METHOD_CONNECT => Request::METHOD_CONNECT,
      ],
      '#default_value' => array_keys($config['methods']),
      '#description' => $this->t('Leave empty to skip method check.'),
    ];
    $form['advanced']['content_type'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Content type (regex)'),
      '#default_value' => $config['content_type'],
      '#placeholder' => '#text/html#',
      '#description' => $this->t('Leave empty to skip content type check.'),
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->cleanValues()->getValues();
    $values['skip_roles'] = Checkboxes::getCheckedCheckboxes($values['skip_roles']);
    $values['methods'] = array_fill_keys(Checkboxes::getCheckedCheckboxes($values['methods']), TRUE);
    $values['skip_usernames'] = array_unique(array_filter(array_map('trim', explode(PHP_EOL, $values['skip_usernames']))));

    $this->config(static::CONFIG_NAME)
      ->setData($values)
      ->save();
    parent::submitForm($form, $form_state);
  }

}
